import uuid

from datetime import datetime
from typing import TYPE_CHECKING

from sqlalchemy import DateTime, ForeignKey, text
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .user import User
    from .voucher import Voucher


@reg.mapped_as_dataclass(unsafe_hash=True)
class Reprint:
    __tablename__ = "reprints"

    id: Mapped[uuid.UUID] = mapped_column(
        "id", UUID(as_uuid=True), primary_key=True, server_default=text("gen_random_uuid()"), insert_default=uuid.uuid4
    )
    date: Mapped[datetime] = mapped_column("date", DateTime, nullable=False, index=True)
    voucher_id: Mapped[uuid.UUID] = mapped_column(
        "voucher_id",
        UUID(as_uuid=True),
        ForeignKey("vouchers.id"),
        nullable=False,
        index=True,
    )
    user_id: Mapped[uuid.UUID] = mapped_column("user_id", UUID(as_uuid=True), ForeignKey("users.id"), nullable=False)

    user: Mapped["User"] = relationship(backref="reprints")
    voucher: Mapped["Voucher"] = relationship(back_populates="reprints")

    def __init__(self, voucher_id=None, user_id=None, id_=None):
        self.id = id_
        self.date = datetime.utcnow()
        self.voucher_id = voucher_id
        self.user_id = user_id
