import barker.schemas.header_footer as schemas

from fastapi import APIRouter, HTTPException, Security, status
from sqlalchemy import select
from sqlalchemy.exc import SQLAlchemyError

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.db_setting import DbSetting
from ..schemas.user_token import UserToken


router = APIRouter()


@router.get("")
def get(
    user: UserToken = Security(get_user, scopes=["owner"]),
):
    with SessionFuture() as db:
        return [
            {"id": item.id, "name": item.name, "text": item.data["Text"]}
            for item in db.execute(select(DbSetting).where(DbSetting.name.in_(["Header", "Footer"]))).scalars().all()
        ]


@router.post("")
def save(
    data: schemas.HeaderFooter,
    user: UserToken = Security(get_user, scopes=["owner"]),
):
    try:
        with SessionFuture() as db:
            item: DbSetting = db.execute(
                select(DbSetting).where(DbSetting.id == data.id_, DbSetting.name == data.name)
            ).scalar_one()
            item.data = {"Text": data.text}
            db.commit()
            return [
                {"id": item.id, "name": item.name, "text": item.data["Text"]}
                for item in db.execute(select(DbSetting).where(DbSetting.name.in_(["Header", "Footer"])))
                .scalars()
                .all()
            ]
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )
