from datetime import date, datetime
from decimal import Decimal

from pydantic import BaseModel, ConfigDict, field_serializer, field_validator

from . import to_camel
from .user import UserLink


class NameAmount(BaseModel):
    name: str
    amount: Decimal
    model_config = ConfigDict(str_strip_whitespace=True)


class SettleOptionSchema(BaseModel):
    def __hash__(self):
        return hash(self.id_)

    def __eq__(self, other):
        return self.id_ == other.id_

    id_: int
    name: str
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True, str_strip_whitespace=True)


class InfoItem(BaseModel):
    date: datetime
    bill_id: str
    customer: str
    amount: Decimal
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date", mode="before")
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")


class CashierReport(BaseModel):
    start_date: date
    finish_date: date
    cashier: UserLink
    cashiers: str
    user: UserLink
    amounts: list[NameAmount]
    info: dict[str, list[InfoItem]]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("start_date", mode="before")
    @classmethod
    def parse_start_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("start_date")
    def serialize_start_date(self, value: date, _info):
        return value.strftime("%d-%b-%Y")

    @field_validator("finish_date", mode="before")
    @classmethod
    def parse_finish_date(cls, value):
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("finish_date")
    def serialize_finish_date(self, value: date, _info):
        return value.strftime("%d-%b-%Y")
