import uuid

from pydantic import BaseModel, ConfigDict, Field

from . import to_camel
from .menu_category import MenuCategoryLink


class ModifierCategoryIn(BaseModel):
    name: str = Field(..., min_length=1)
    minimum: int = Field(ge=0)
    maximum: int | None = Field(ge=0)
    is_active: bool
    menu_categories: list[MenuCategoryLink]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class ModifierCategory(ModifierCategoryIn):
    id_: uuid.UUID
    sort_order: int
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class ModifierCategoryBlank(ModifierCategoryIn):
    name: str
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class ModifierCategoryLink(BaseModel):
    id_: uuid.UUID = Field(...)
    name: str | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)
