import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';

import { Printer } from '../../core/printer';
import { ToasterService } from '../../core/toaster.service';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { PrinterService } from '../printer.service';

@Component({
  selector: 'app-printer-detail',
  templateUrl: './printer-detail.component.html',
  styleUrls: ['./printer-detail.component.css'],
})
export class PrinterDetailComponent implements OnInit, AfterViewInit {
  @ViewChild('nameElement', { static: true }) nameElement?: ElementRef;
  form: FormGroup<{
    name: FormControl<string>;
    address: FormControl<string>;
    cutCode: FormControl<string>;
  }>;

  item: Printer = new Printer();

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private toaster: ToasterService,
    private ser: PrinterService,
  ) {
    // Create form
    this.form = new FormGroup({
      name: new FormControl<string>('', { nonNullable: true }),
      address: new FormControl<string>('', { nonNullable: true }),
      cutCode: new FormControl<string>('', { nonNullable: true }),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { item: Printer };
      this.showItem(data.item);
    });
  }

  showItem(item: Printer) {
    this.item = item;
    this.form.setValue({
      name: this.item.name ?? '',
      address: this.item.address ?? '',
      cutCode: this.item.cutCode ?? '',
    });
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.nameElement !== undefined) {
        this.nameElement.nativeElement.focus();
      }
    }, 0);
  }

  save() {
    this.ser.saveOrUpdate(this.getItem()).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/printers');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  delete() {
    this.ser.delete(this.item.id as string).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/printers');
      },
      (error) => {
        this.toaster.show('Error', error);
      },
    );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: { title: 'Delete Printer?', content: 'Are you sure? This cannot be undone.' },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): Printer {
    const formModel = this.form.value;
    this.item.name = formModel.name;
    this.item.address = formModel.address;
    this.item.cutCode = formModel.cutCode;
    return this.item;
  }
}
